(document:surround "/std/frame")
(document:insert "/std/functions")

(define wizardface (make-widget "wizardface"))
(define *steps-file* (global 'steps-file))

;;; Functions
(define *step* (make-cell 0))
(define *step-list* (make-cell '()))

(define (step) (cell-ref *step*))
(define (++step) (cell-set! *step* (+ (step) 1)) (step))
(define (--step) (cell-set! *step* (- (step) 1)) (step))
(define (last) (- (length (cell-ref *step-list*)) 1))

(define (url) (car (list-ref (cell-ref *step-list*) (step))))
(define (topic) (cdr (list-ref (cell-ref *step-list*) (step))))

(define (sure-yes key name list)
  (or (member key list)
      (wizard-id action-add (vector key name))))

(define (sure-no key name list)
  (and (member key list)
       (wizard-id action-remove key)))

(define (action-name x) (vector-ref x 0))

(define (check-buttons step last)
  (with-translation _ "alterator-wizard"
                    (let ((buttons (map action-name (wizard-id actions))))
                      (cond
                       ((= step 0)
                        (sure-yes 'forward (_ "Next") buttons)
                        (sure-no 'backward (_ "Previous") buttons)
                        (sure-no 'finish (_ "Finish") buttons))
                       ((< 0 step last)
                        (sure-yes 'forward (_ "Next") buttons)
                        (sure-yes 'backward (_ "Previous") buttons)
                        (sure-no 'finish (_ "Finish") buttons))
                       ((= step last)
                        (sure-no 'forward (_ "Next") buttons)
                        (sure-yes 'backward (_ "Previous") buttons)
                        (sure-yes 'finish (_ "Finish") buttons))))
                    #t))

(define (wizard:replace direction)
  (check-buttons (step) (last))
  (frame:replace (url) 'frame:direction direction)
  (document:root
    help (woo-get-option (woo-read-first "/help/" 'topic (topic)) 'url)))

(define (go-next light)
  (and (< (step) (last))
       (or light (not (eq? (wizard-call "next") 'cancel)))
       (wizard-id current-step (++step))
       (wizard:replace 'next)))

(define (go-back light)
  (and (> (step) 0)
       (or light (not (eq? (wizard-call "previous") 'cancel)))
       (wizard-id current-step (--step))
       (wizard:replace 'back)))

(define (go-end light)
  (and (or light (not (eq?  (wizard-call "next") 'cancel)))
       (document:end)))

(define (name+help x)
  (cons (woo-get-option x 'name)
        (woo-get-option x 'help)))

(define (label+icon x)
  (cons (woo-get-option x 'label)
        (woo-get-option x 'icon)))

;;; UI
(frame:set-next (lambda() (go-next #t)))
(frame:set-back (lambda() (go-back #t)))
(frame:set-end  (lambda() (go-end #t)))

title (_ "Installation")

(document:id wizard-id
  (frame:set-wizard
    (wizardface
      title (_ "Installation")
      name "wizardface"
      (frame:page)
      (when clicked
        (case (wizard-id current-action)
          ((finish) (go-end #f))
          ((forward) (go-next #f))
          ((backward) (go-back #f)))))))

;;; Logic
(document:root
 (when loaded
   (catch/message
     (lambda()
       (wizardface-context-set!)
       (let ((step-list (woo-list "/step-list" 'steps_file *steps-file*)))
	 (cell-set! *step-list* (map name+help step-list))
	 (wizard-id steps (map label+icon step-list)
		    current-step 0)
	 (wizard-id action-add 'help (_ "Help"))
	 (wizard:replace 'next))))))

